@@ -19,6 +19,7 @@ limitations under the License.
 local E = {}
 local fun = require "fun"
 local util = require "rspamd_util"
+local rspamd_parsers = require "rspamd_parsers"
 local rspamd_regexp = require "rspamd_regexp"
 local rspamd_lua_utils = require "lua_util"
 
@@ -55,51 +56,70 @@ rspamd_config.R_PARTS_DIFFER = {
 }
 
 -- Date issues
-rspamd_config.MISSING_DATE = {
+local date_id = rspamd_config:register_symbol({
+  name = 'DATE_CB',
+  type = 'callback,mime',
   callback = function(task)
-    local date = task:get_header_raw('Date')
-    if date == nil or date == '' then
-      return true
+    local date_time = task:get_header('Date')
+    if date_time == nil or date_time == '' then
+      task:insert_result('MISSING_DATE', 1.0)
+      return
     end
-    return false
-  end,
+
+    local dm, err = rspamd_parsers.parse_smtp_date(date_time)
+    if err then
+      task:insert_result('INVALID_DATE', 1.0)
+      return
+    end
+
+    local dt = task:get_date({format = 'connect', gmt = true})
+    local date_diff = dt - dm
+
+    if date_diff > 86400 then
+      -- Older than a day
+      task:insert_result('DATE_IN_PAST', 1.0, tostring(math.floor(date_diff/3600)))
+    elseif -date_diff > 7200 then
+      -- More than 2 hours in the future
+      task:insert_result('DATE_IN_FUTURE', 1.0, tostring(math.floor(-date_diff/3600)))
+    end
+  end
+})
+
+rspamd_config:register_symbol({
+  name = 'MISSING_DATE',
   score = 1.0,
   description = 'Message date is missing',
   group = 'headers',
-  type = 'mime',
-}
+  type = 'virtual',
+  parent = date_id,
+})
 
-rspamd_config.DATE_IN_FUTURE = {
-  callback = function(task)
-    local dm = task:get_date{format = 'message', gmt = true}
-    local dt = task:get_date{format = 'connect', gmt = true}
-    -- 2 hours
-    if dm > 0 and dm - dt > 7200 then
-      return true
-    end
-    return false
-  end,
+rspamd_config:register_symbol({
+  name = 'INVALID_DATE',
+  score = 1.5,
+  description = 'Malformed date header',
+  group = 'headers',
+  type = 'virtual',
+  parent = date_id,
+})
+
+rspamd_config:register_symbol({
+  name = 'DATE_IN_FUTURE',
   score = 4.0,
   description = 'Message date is in future',
   group = 'headers',
-  type = 'mime',
-}
+  type = 'virtual',
+  parent = date_id,
+})
 
-rspamd_config.DATE_IN_PAST = {
-  callback = function(task)
-    local dm = task:get_date{format = 'message', gmt = true}
-    local dt = task:get_date{format = 'connect', gmt = true}
-    -- A day
-    if dm > 0 and dt - dm > 86400 then
-      return true
-    end
-    return false
-  end,
+rspamd_config:register_symbol({
+  name = 'DATE_IN_PAST',
   score = 1.0,
   description = 'Message date is in past',
   group = 'headers',
-  type = 'mime',
-}
+  type = 'virtual',
+  parent = date_id,
+})
 
 local obscured_id = rspamd_config:register_symbol{
   callback = function(task)
@@ -521,7 +541,7 @@ local check_from_display_name = rspamd_config:register_symbol{
     local from = task:get_from(2)
     if not (from and from[1] and from[1].name) then return false end
     -- See if we can parse an email address from the name
-    local parsed = util.parse_mail_address(from[1].name, task:get_mempool())
+    local parsed = rspamd_parsers.parse_mail_address(from[1].name, task:get_mempool())
     if not parsed then return false end
     if not (parsed[1] and parsed[1]['addr']) then return false end
     -- Make sure we did not mistake e.g. <something>@<name> for an email address
@@ -602,7 +622,7 @@ rspamd_config.SPOOF_REPLYTO = {
     end
     if not found_fromdom then return false end
     -- Parse Reply-To header
-    local parsed = ((util.parse_mail_address(rt, task:get_mempool()) or E)[1] or E).domain
+    local parsed = ((rspamd_parsers.parse_mail_address(rt, task:get_mempool()) or E)[1] or E).domain
     if not parsed then return false end
     -- Reply-To domain must be different to From domain
     if not util.strequal_caseless(parsed, from[1].domain) then
@@ -618,8 +638,9 @@ rspamd_config.SPOOF_REPLYTO = {
 
 rspamd_config.INFO_TO_INFO_LU = {
   callback = function(task)
-    local lu = task:get_header('List-Unsubscribe')
-    if not lu then return false end
+    if not task:has_header('List-Unsubscribe') then
+      return false
+    end
     local from = task:get_from('mime')
     if not (from and from[1] and util.strequal_caseless(from[1].user, 'info')) then
       return false
